/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Test per els Dao
 *
 * @author Macià Sintes Andreu
 */

package org.dobile.db.dao;

import androidx.arch.core.executor.testing.InstantTaskExecutorRule;
import androidx.room.Room;
import androidx.lifecycle.Observer;
import android.content.Context;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.ext.junit.runners.AndroidJUnit4;
//import androidx.test.runner.AndroidJUnit4;
import android.util.Log;

import org.dobile.db.DobileRoomDatabase;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.Scene;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestRule;
import org.junit.runner.RunWith;
import org.mockito.Mock;


import java.io.IOException;
import java.util.List;

import static org.junit.Assert.*;


/**
 * Test per la classe SceneDao
 */
@RunWith(AndroidJUnit4.class)
public class SceneDaoTest {
    private DobileRoomDatabase db;
    private SceneDao sceneDao;

    @Rule
    public TestRule rule = new InstantTaskExecutorRule();


    @Mock
    private Observer<List<DeviceSwitch>> observer;

    /**
     * Crea una base de dades en memòria per fer els test i  no afectar a les dades guardades a la base
     * de dades persistent. Les dades creades en aquesta base de dades desapareixen quan finalitza el
     * proces.
     * Room comprova que no s'accedeix a la base de dades des del thread principal, no hi deixa accedir
     * des d'aquest thread. Per fer les proves es deshabilita aquesta comprovació amb alloMainThreadQueries()
     */
    @Before
    public  void createDb(){

        //MockitoAnnotations.initMocks(this);

        Context context = InstrumentationRegistry.getInstrumentation().getTargetContext();
        db = Room.inMemoryDatabaseBuilder(context, DobileRoomDatabase.class)
                .allowMainThreadQueries().build();
        sceneDao = db.sceneDao();
    }

    /**
     * Tanca la base de dades
     *
     * @throws IOException
     */
    @After
    public void closeDb() throws IOException {
        db.close();
    }

    @Test
    public void insertTest() throws Exception {
        Scene scene = new Scene("Salonppps");
        sceneDao.insert(scene);



        List<Scene> listScenes = sceneDao.getListAllScenes();
        Log.d("listScenes tamany: ", "Nombre escenes: " + listScenes.size());

        assertEquals(1, listScenes.size());

    }

    @Test
    public void deleteAllTest() throws Exception{

        Scene scene = new Scene("Pati");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        Log.d("deleteAllTest -listScenes tamany: ", "Nombre escenes abans esborrar: " + listScenes.size());

        sceneDao.deleteAll();
        listScenes = sceneDao.getListAllScenes();

        Log.d("deleteAllTest - listScenes tamany: ", "Nombre escenes despres  esborrar: " + listScenes.size());

        assertEquals(0, listScenes.size());

    }

    @Test
    public void deleteTest() throws Exception{

        Scene scene = new Scene("Pati2");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        Log.d("deleteTest - listScenes tamany: ", "Nombre escenes abans esborrar: " + listScenes.size());

        Scene scene2 = listScenes.get(0);
        sceneDao.deleteScene(scene2);

        listScenes = sceneDao.getListAllScenes();

        Log.d("deleteTest -  listScenes tamany: ", "Nombre escenes despres  esborrar: " + listScenes.size());

        assertEquals(0, listScenes.size());

    }

    @Test
    public void updateTest() throws Exception{

        Scene scene3 = new Scene("Terraca");
        sceneDao.insert(scene3);

        List<Scene> listScenes = sceneDao.getListAllScenes();

        Scene scene2 = listScenes.get(0);
        scene2.setName("Segon pis");
        sceneDao.update(scene2);

        listScenes = sceneDao.getListAllScenes();
        Scene scene_updated = listScenes.get(0);

        assertEquals("Segon pis", scene_updated.getName());

    }

    @Test
    public void getSceneTest() throws Exception{
        Scene scene = new Scene("Salon");
        sceneDao.insert(scene);

        List<Scene> listScenesAll = sceneDao.getListAllScenes();
        Scene sceneWithId = listScenesAll.get(0);

        List<Scene> listScenesSelected = sceneDao.getScene(sceneWithId.getId());
        Scene sceneSelected = listScenesSelected.get(0);

        assertEquals(sceneWithId.getId(), sceneSelected.getId());
    }

}
