/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Test per els Dao
 *
 * @author Macià Sintes Andreu
 */

package org.dobile.db.dao;

import androidx.arch.core.executor.testing.InstantTaskExecutorRule;
import androidx.room.Room;
import android.content.Context;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.ext.junit.runners.AndroidJUnit4;

import org.dobile.db.DobileRoomDatabase;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;
import org.dobile.db.entity.Scene;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestRule;
import org.junit.runner.RunWith;
import org.mockito.MockitoAnnotations;


import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

/**
 * Test per la classe SwitchNamesDao
 */
@RunWith(AndroidJUnit4.class)
public class SwitchNamesDaoTest {
    private DobileRoomDatabase db;
    private SwitchNamesDao switchNamesDao;
    private SceneDao sceneDao;
    private DeviceSwitchDao deviceSwitchDao;

    @Rule
    public TestRule rule = new InstantTaskExecutorRule();

    /**
     * Crea una base de dades en memòria per fer els test i  no afectar a les dades guardades a la base
     * de dades persistent. Les dades creades en aquesta base de dades desapareixen quan finalitza el
     * proces.
     * Room comprova que no s'accedeix a la base de dades des del thread principal, no hi deixa accedir
     * des d'aquest thread. Per fer les proves es deshabilita aquesta comprovació amb alloMainThreadQueries()
     */
    @Before
    public  void createDb(){

        MockitoAnnotations.initMocks(this);

        Context context = InstrumentationRegistry.getInstrumentation().getTargetContext();
        db = Room.inMemoryDatabaseBuilder(context, DobileRoomDatabase.class)
                .allowMainThreadQueries().build();
        switchNamesDao = db.switchNamesDao();
        sceneDao = db.sceneDao();
        deviceSwitchDao = db.deviceSwitchDao();

    }

    /**
     * Tanca la base de dades
     *
     * @throws IOException
     */
    @After
    public void closeDb() throws IOException {
        db.close();
    }

    @Test
    public void insertDeviceName() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch devFocus = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(devFocus);
        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        int idDevice = listDevices.get(0).getId();

        SwitchNames devName = new SwitchNames("Focus cuina", idDevice);
        switchNamesDao.insert(devName);

        List<SwitchNames> listDevNames = switchNamesDao.getListAllSwitchNames();
        assertEquals(1, listDevNames.size());
    }

    @Test
    public void insertListDeviceName() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch devFocus = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(devFocus);
        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        int idDevice = listDevices.get(0).getId();

        SwitchNames dev2Name1 = new SwitchNames("Focus cuina 1", idDevice);
        SwitchNames dev2Name2 = new SwitchNames("Focus cuina 2", idDevice);
        SwitchNames dev2Name3 = new SwitchNames("Focus cuina 3", idDevice);

        List<SwitchNames> listDevNamesToInsert = new ArrayList<>();
        listDevNamesToInsert.add(dev2Name1);
        listDevNamesToInsert.add(dev2Name2);
        listDevNamesToInsert.add(dev2Name3);

        switchNamesDao.insert(listDevNamesToInsert);


        List<SwitchNames> listDevNames = switchNamesDao.getListAllSwitchNames();
        assertEquals(3, listDevNames.size());
    }

    @Test
    public void delete() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch devFocus = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(devFocus);
        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        int idDevice = listDevices.get(0).getId();

        SwitchNames devName = new SwitchNames("Focus 1", idDevice);
        switchNamesDao.insert(devName);

        List<SwitchNames> listDevNames = switchNamesDao.getListAllSwitchNames();
        assertEquals("Noms dispositiu abans esborrar igual a 1", 1, listDevNames.size());

        SwitchNames devSelected = listDevNames.get(0);
        switchNamesDao.deleteSwitchNames(devSelected);

        List<SwitchNames> listDevNamesDeleted = switchNamesDao.getListAllSwitchNames();
        assertEquals("Noms dispositiu despres esborrar igual a 0", 0, listDevNamesDeleted.size());

    }

    @Test
    public void update() throws Exception {
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        DeviceSwitch devFocus = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames);

        deviceSwitchDao.insertDevice(devFocus);
        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        int idDevice = listDevices.get(0).getId();

        SwitchNames dev3Name1 = new SwitchNames("Focus cuina 1", idDevice);
        SwitchNames dev3Name2 = new SwitchNames("Focus cuina 2", idDevice);
        SwitchNames dev3Name3 = new SwitchNames("Focus cuina 3", idDevice);

        List<SwitchNames> listDevNamesToInsert = new ArrayList<>();
        listDevNamesToInsert.add(dev3Name1);
        listDevNamesToInsert.add(dev3Name2);
        listDevNamesToInsert.add(dev3Name3);

        switchNamesDao.insert(listDevNamesToInsert);

        List<SwitchNames> listDevNames = switchNamesDao.getListAllSwitchNames();
        assertEquals("Nom dispositiu es Focus cuina 3", "Focus cuina 3", listDevNames.get(2).getName());

        SwitchNames devSelected = listDevNames.get(2);
        devSelected.setName("Llum central");
        switchNamesDao.updateSwitchNames(listDevNames);

        List<SwitchNames> listDevNamesUpdated = switchNamesDao.getListAllSwitchNames();
        assertEquals("Nom dispositiu canviat a Llum central", "Llum central", listDevNamesUpdated.get(2).getName());

    }

    @Test
    public void getListNamesForDevice() throws Exception{
        Scene scene = new Scene("Cuina");
        sceneDao.insert(scene);

        List<Scene> listScenes = sceneDao.getListAllScenes();
        scene = listScenes.get(0);

        List<SwitchNames> deviceNames = new ArrayList<>();

        SwitchNames devName1 = new SwitchNames("Focus 1");
        SwitchNames devName2 = new SwitchNames("Focus 2");
        SwitchNames devName3 = new SwitchNames("Focus 3");

        deviceNames.add(devName1);
        deviceNames.add(devName2);
        deviceNames.add(devName3);

        List<SwitchNames> deviceNames2 = new ArrayList<>();

        SwitchNames dev2Name1 = new SwitchNames("Focus 1");
        SwitchNames dev2Name2 = new SwitchNames("Focus 2");
        SwitchNames dev2Name3 = new SwitchNames("Focus 3");

        deviceNames2.add(dev2Name1);
        deviceNames2.add(dev2Name2);
        deviceNames2.add(dev2Name3);

        DeviceSwitch devLampara = new DeviceSwitch("Lampara cuina", "192.168.1.156", 3, scene.getId(), deviceNames);
        DeviceSwitch devFocus = new DeviceSwitch("Focus cuina", "192.168.1.157", 3, scene.getId(), deviceNames2);

        deviceSwitchDao.insertDevice(devLampara);
        deviceSwitchDao.insertDevice(devFocus);

        List<DeviceSwitch> listDevices = deviceSwitchDao.getListAllDevices();

        if ( listDevices.get(0).getName() == "Lampara cuina"){
            devLampara = listDevices.get(0);
            devFocus = listDevices.get(1);
        }else {
            devLampara = listDevices.get(1);
            devFocus = listDevices.get(0);
        }

        int idDeviceLampara = devLampara.getId() ;
        int idDeviceFocus = devFocus.getId();

        SwitchNames dev4Name1 = new SwitchNames("Focus cuina 1", idDeviceLampara);
        SwitchNames dev4Name2 = new SwitchNames("Focus cuina 2", idDeviceLampara);
        SwitchNames dev4Name3 = new SwitchNames("Focus cuina 3", idDeviceLampara);

        SwitchNames dev4Name4 = new SwitchNames("Lampara central", idDeviceFocus);

        List<SwitchNames> listDevNamesToInsert = new ArrayList<>();
        listDevNamesToInsert.add(dev4Name1);
        listDevNamesToInsert.add(dev4Name2);
        listDevNamesToInsert.add(dev4Name3);
        listDevNamesToInsert.add(dev4Name4);

        switchNamesDao.insert(listDevNamesToInsert);

        List<SwitchNames> listDevNames = switchNamesDao.getListAllSwitchNames();
        assertEquals("Llista de tots els noms de dispositius igual a 4", 4, listDevNames.size());

        List<SwitchNames> listDevNamesLampara = switchNamesDao.getListNamesForDevice(idDeviceLampara);
        assertEquals("Llista dels noms del dispositiu de lampara igual a 3", 3, listDevNamesLampara.size());

    }
}
