/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 * DAO amb els mètodes per accedir a la taula dels escenaris
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.db.dao;

import java.util.List;

import androidx.lifecycle.LiveData;
import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.OnConflictStrategy;
import androidx.room.Query;
import androidx.room.Update;

import org.dobile.db.entity.Scene;

/**
 * DAO amb els mètodes per accedir a la taula dels escenaris
 */
@Dao
public interface SceneDao {

    /**
     * Inserta una escena
     *
     * @param scene
     */
    @Insert
    void insert(Scene scene);

    /**
     * Actualitza una escena
     *
     * @param scene
     */
    @Update(onConflict = OnConflictStrategy.REPLACE)
    void update(Scene scene);

    /**
     * Esborra una escena
     *
     * @param scene
     */
    @Delete
    void deleteScene(Scene scene);

    /**
     * Esborra totes les escenes
     */
    @Query("DELETE  FROM scenes")
    void deleteAll();


    /**
     * Retorna totes les escenes en un LiveData
     *
     * @return LiveData amb totes les escenes
     */
    @Query("SELECT * from scenes ORDER BY name ASC")
    LiveData<List<Scene>> getAllScenes();

    /**
     * Retorna totes les escenes en una List, per facilitar els tests
     *
     * @return List amb totes les escenes
     */
    @Query("SELECT * from scenes ORDER BY name ASC")
    List<Scene> getListAllScenes();

    /**
     * Retorna una escena amb id demanat.
     * @param id
     * @return List amb una escena amb id demanat
     */
    @Query("SELECT * from scenes WHERE id IS :id LIMIT 1")
    List<Scene> getScene(int id);


}
