
/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 *
 * Entitat amb la taula dels dispositius
 *
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.entity;

import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.PrimaryKey;
import androidx.annotation.NonNull;
import java.io.Serializable;


/**
 * Entity que representa la taula devices_ac
 */
@Entity(tableName = "devices_ac")
public class DeviceAC implements Serializable {

    /**
     * Mode funcionament de l'aire acondicionat en fred
     */
    public static int COOL = 0;

    /**
     * Mode funcionament de l'aire acondicionat en calor
     */
    public static  int HEAT = 1;

    /**
     * Clau primària id, generada automàticament
     */
    @PrimaryKey(autoGenerate = true)
    private int id;

    /**
     * Nom del dispositiu
     */
    @NonNull
    @ColumnInfo(name = "name")
    private String name;

    /**
     * Ip del dispositiu
     */
    @NonNull
    @ColumnInfo(name = "ip")
    private String ip;

    /**
     *  Darrer mode de funcioament de l'aire acondicionat, fred o calor, elegit per l'usuari
     *
     *      - 0 = mode fred
     *      - 1 = mode calor
     */
    @ColumnInfo(name = "mode")
    private int mode;

    /**
     * Darrera temperatura seleccionada a l'equip d'aire acondicionat
     */
    @ColumnInfo(name = "temperature")
    private int temperature;

    /**
     * Id de l'escena a la qual pertany el dispositiu
     */
    @ColumnInfo(name = "scene_id")
    private int sceneId;

    /**
     * Estat de l'interruptor ON/OFF
     *
     *  0 = OFF
     *  1 = ON
     */
    @ColumnInfo (name = "on")
    private int on;


    /**
     * Constructor del dispositiu
     *
     * @param name
     * @param ip
     * @param mode
     * @param temperature
     * @param sceneId
     */
    public DeviceAC(String name, String ip, int mode, int temperature, int sceneId ) {
        this.name = name;
        this.ip = ip;
        this.setMode(mode);
        this.setTemperature(temperature);
        this.sceneId = sceneId;
    }


    /**
     * Retorna l'id del dispositiu
     *
     * @return Id del dispositiu
     */
    public int getId() {
        return id;
    }

    /**
     * Canvia l'id del dispositiu
     *
     * @param id
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Retorna el nom del dispositiu
     *
     * @return Nom del dispositiu
     */
    public String getName(){return this.name;}

    /**
     * Canvia el nom del dispositiu
     *
     * @param name
     */
    public void setName(@NonNull String name) {
        this.name = name;
    }

    /**
     * Retorna la ip del dispositiu
     *
     * @return Ip del dispositiu
     */
    @NonNull
    public String getIp() {
        return ip;
    }

    /**
     * Canvia la ip del dispositiu
     *
     * @param ip
     */
    public void setIp(@NonNull String ip) {
        this.ip = ip;
    }


    /**
     * Retorna l'id de l'escena a la qual pertany el dispositiu
     *
     * @return Id de l'escena a la qual pertany el dispositiu
     */
    public int getSceneId() {
        return sceneId;
    }

    /**
     * Canvia l'id de l'escena a la qual pertany el dispositiu
     *
     * @param sceneId
     */
    public void setSceneId(int sceneId) {
        this.sceneId = sceneId;
    }


    /**
     * Mode funcionament de l'aire acondicionat, fred o calor
     */
    public int getMode() {
        return mode;
    }

    /**
     *
     * @param mode
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    /**
     * Darrera temperatura a la qual s'ha ajustat l'equip d'aire acondiconada
     */
    public int getTemperature() {
        return temperature;
    }

    /**
     *
     * @param temperature
     */
    public void setTemperature(int temperature) {
        this.temperature = temperature;
    }

    /**
     * Getter de on
     *
     * @return Darrer estat de on
     */
    public int getOn() {
        return on;
    }

    /**
     * Setter de on
     *
     * @param on
     */
    public void setOn(int on) {
        this.on = on;
    }
}