/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 *
 * Entitat amb la taula dels noms dels escenaris
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.entity;



import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.PrimaryKey;
import androidx.annotation.NonNull;

import java.io.Serializable;


/**
 * Entity que representa la taula scenes, que conté les escenes
 */
@Entity(tableName = "scenes")
public class Scene implements Serializable {

    /**
     * Clau primària id, generada automàticament
     */
    @PrimaryKey(autoGenerate = true)
    private int id;

    /**
     * Nom de l'escena
     */
    @NonNull
    @ColumnInfo(name = "name")
    private String name;

    /**
     * Constructor de l'escena amb el nom
     * @param name
     */
    public Scene(@NonNull String name) {
        this.name = name;
    }

    /**
     * Retorna l'id de l'escena
     *
     * @return Id de l'escena
     */
    public int getId() { return this.id; }

    /**
     * Canvia l'id de l'escena
     *
     * @param id
     */
    public void setId(int id) { this.id = id;}

    /**
     * Retorna el nom de l'escena
     *
     * @return Nom de l'escena
     */
    public String getName(){
        return this.name;
    }

    /**
     * Canvia el nom de l'escena
     *
     * @param name
     */
    public void setName(String name) { this.name = name;}
}