/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Per la persistència s'utilitza la llibreria Room. Aquesta llibreria aporta una capa d'abstracció sobre SQLite. Conté tres
 * componentes principals:
 *
 *  - Database: És el punt d'acces principal a la base de dades
 *  - Entity: Representa una taula
 *  - DAO: Conté els mètodes per accedir a la base de dades
 *
 *
 * Entitat amb la taula dels noms d'interruptor
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.db.entity;



import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.ForeignKey;
import androidx.room.Ignore;
import androidx.room.PrimaryKey;

import org.dobile.model.DeviceName;

import java.io.Serializable;


/**
 * Entity que representa la taula device_names que conté els noms d'interruptors dels
 * dispositius
 */
@Entity (tableName = "switch_names",
        foreignKeys =
        @ForeignKey(entity = DeviceSwitch.class,
                parentColumns = "id",
                childColumns = "device_id",
                onDelete = ForeignKey.CASCADE))
public class SwitchNames implements DeviceName, Serializable {

    /**
     * Clau primària id, generada automàticament
     */
    @PrimaryKey(autoGenerate = true)
    private int id;

    /**
     * Nom de l'interruptor
     */
    @ColumnInfo(name = "name")
    private String name;

    /**
     * Id del dispositiu al qual pertany l'interruptor
     */
    @ColumnInfo(name = "device_id")
    private int deviceId;

    /**
     * Constructor del nom d'interruptor
     *
     * @param name
     */
    public SwitchNames(String name){
        this.name = name;
    }

    /**
     * Constructor del nom d'interruptor amb nom i id
     *
     * @param name
     * @param deviceId
     */
    @Ignore
    public SwitchNames(String name, int deviceId){
        this.name = name;
        this.deviceId = deviceId;
    }

    /**
     * Retorna l'id del nom d'interruptor
     *
     * @return Id del nom d'interruptor
     */
    public int getId() {
        return id;
    }

    /**
     * Retorna el nom de l'interruptor
     *
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * Retorna l'id del dispositiu al qual pertany l'interruptor
     *
     * @return
     */
    public int getDeviceId() {
        return deviceId;
    }

    /**
     * Canvia l'id del nom d'interruptor
     *
     * @param id
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Canvia el nom de l'interruptor
     *
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Canvia l'id del dispositiu al qual pertany l'interruptor
     * @param deviceID
     */
    public void setDeviceId(int deviceID) {
        this.deviceId = deviceID;
    }
}

