/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Adapter per del RecyclerView per mostrar la llista de dispositius
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;


import android.content.Context;
import android.content.Intent;
import androidx.recyclerview.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import org.dobile.R;
import org.dobile.db.entity.DeviceSwitch;

import java.util.List;

/**
 * ListAdapter per mostrar la llista de dispositius d'una escena
 */
public class DevSwitchesListAdapter extends RecyclerView.Adapter<DevSwitchesListAdapter.DeviceViewHolder> {
    private final LayoutInflater mInflater;

    /*Llista dels dispositius*/
    private List<DeviceSwitch> mDevices;

    /**
     * Classe interna ViewHolder que representa un sol ítem de la llista amb les dades a mostrar, en aquest un dispositiu de l'escena
     */
    class DeviceViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        private final TextView deviceItemView;
        final DevSwitchesListAdapter mAdapter;

        /**
         * Constructor del ViewHolder
         *
         * @param itemView
         * @param adapter
         */
        private DeviceViewHolder(View itemView, DevSwitchesListAdapter adapter) {
            super(itemView);
            deviceItemView = itemView.findViewById(R.id.textDeviceItemSwitch);
            this.mAdapter = adapter;
            deviceItemView.setOnClickListener(this);
        }

        /**
         * Acció que s'executa quan es clicka un ítem de la llista
         *
         * @param v View del item clickat
         */
        @Override
        public void onClick(View v) {
            //Es recupera la posició de l'ítem clickat
            int mPosition = getLayoutPosition();

            //Amb la posició s'extreu el dispositiu de la llista de dispositius
            DeviceSwitch device = mDevices.get(mPosition);



            //Es crea un nou intent per mostrar el dispositiu seleccionat.
            //Aquest intent arranca una activity DeviceSwitchActivity o
            //DeviceACActivity depenent del tipus de dispositiu seleccionat
            Context context = v.getContext();
            Intent intent;
            intent = new Intent(context, DeviceSwitchActivity.class);
            intent.putExtra("deviceSelected", device);
            context.startActivity(intent);

        }
    }

    /**
     * Constructor de DevicesListAdapter
     *
     * @param context
     */
    DevSwitchesListAdapter(Context context) {
        mInflater = LayoutInflater.from(context);
    }

    /**
     * Crea un nou ViewHolder inflant el layout device_switch_item que contindrà un sol ítem
     *
     * @param parent
     * @param viewType
     * @return
     */
    @Override
    public DeviceViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = mInflater.inflate(R.layout.device_switch_item, parent, false);
        return new DeviceViewHolder(itemView, this);
    }

    /**
     * Assigna les dades, en aquest cas el nom del dispositiu, a l'ítem que està en una determinada posició
     *
     * @param holder ViewHolder de l'ítem
     * @param position Posició de l'ítem a la llista
     */
    @Override
    public void onBindViewHolder(DeviceViewHolder holder, int position) {
        if (mDevices != null) {
            DeviceSwitch current = mDevices.get(position);
            holder.deviceItemView.setText(current.getName());

        }
    }

    /**
     * Assigna la llista de dispositius i notifica a l'Observer que les dades han canviat
     * @param devices
     */
    void setDevices(List<DeviceSwitch> devices){
        mDevices = devices;
        notifyDataSetChanged();
    }

    /**
     * Retorna el número de dispositius que conté la llista
     *
     * @return El número de dispositius de la llista o 0 si la llista és null
     */
    @Override
    public int getItemCount() {
        if (mDevices != null)
            return mDevices.size();
        else return 0;
    }

    /**
     * Retorna el dispositiu d'una posició determinada de la llista
     *
     * @param position Posició del dispositiu a la llista
     * @return Dispositiu de la posició demanada
     */
    public DeviceSwitch getDeviceAtPosition (int position) {
        return mDevices.get(position);
    }
}