/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat que envia els comandaments de tipus emisor infrarrojos. En aquest cas controla un equip d'aire acondicionat
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;

import androidx.annotation.Nullable;
import androidx.appcompat.view.menu.MenuBuilder;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProviders;
import android.content.Context;
import android.content.Intent;
import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.NumberPicker;
import androidx.appcompat.widget.Toolbar;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;

import org.dobile.R;
import org.dobile.db.entity.DeviceAC;
import org.dobile.network.DobileRequestQueue;
import org.dobile.viewmodel.DeviceACViewModel;

/**
 * Classe que envia els comandaments de un dispositiu de tipus aire acondicionat
 */
public class DeviceACActivity extends AppCompatActivity {

    private static final String LOG_TAG = DeviceSwitchActivity.class.getSimpleName();
    public static final String DEVICE = "org.dobile.ui.deviceSwitchActivity.extra.DEVICE";

    private final int ON = 0;
    private final int OFF = 1;
    private final int UPDATE = 2;
    private final int STATUS = 3;

    //Entity del dispositiu
    private DeviceAC device;

    //ViewModel
    private DeviceACViewModel deviceACViewModel;


    private NumberPicker npTemp;
    private Switch swOn;
    private Switch swMode;
    private TextView textTemperature;
    private boolean started = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_device_ac);
        Toolbar toolbar = findViewById(R.id.toolbarDeviceAC);
        setSupportActionBar(toolbar);

        //Mostra el butó Enrera per tornar a la pantalla de l'escena
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //Listener pel butó Enrera. Finalitza l'activity
        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        deviceACViewModel = ViewModelProviders.of(this).get(DeviceACViewModel.class);

        //A través de l'intent recupera el dispositiu seleccionat i assigna el seu nom al títol
        Intent intent = getIntent();
        device = (DeviceAC)intent.getSerializableExtra("deviceSelected");
        Log.d(LOG_TAG, "Dispositiu seleccionat - id: " + device.getId());

        getSupportActionBar().setTitle(device.getName());


        //Recupera les views i assigna els listeners
        npTemp = findViewById(R.id.numberPickerTemperature);
        swOn = findViewById(R.id.switchACOn);
        swMode = findViewById(R.id.switchACMode);
        textTemperature = findViewById(R.id.textViewTemperature);

        swOn.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

                if (isChecked){
                    sendCommand(ON);
                    started = true;
                    device.setOn(1);
                }else{
                    sendCommand(OFF);
                    started = false;
                    device.setOn(0);
                }

                deviceACViewModel.update(device);


            }
        });

        swMode.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {


                //Desa a la base de dades el darrer estat de mode ajustat
                if(isChecked){
                    device.setMode(1);
                }else{
                    device.setMode(0);
                }
                deviceACViewModel.update(device);

                //Si l'equip esta arrancat envia la comanda
                if (started){
                    sendCommand(UPDATE);
                }

            }
        });



        npTemp.setMaxValue(24);
        npTemp.setMinValue(18);

        npTemp.setOnValueChangedListener(new NumberPicker.OnValueChangeListener() {
            @Override
            public void onValueChange(NumberPicker numberPicker, int oldVal, int newVal) {

                //Desa a la base de dades el darrer valor de temperatura ajustada
                device.setTemperature(newVal);
                deviceACViewModel.update(device);

                //Si l'equip esta arrancat envia la comanda
                if (started){
                    sendCommand(UPDATE);
                }



            }
        });


        //Es crea un Observer sobre el LiveData del dispositiu. Actualitza la vista si és necessari amb el
        //darrer estat del mode funcionament i temperatura ajustades
        setObserver();

    }

    /**
     * S'envia una comanda per demanar la temperatura del sensor. S'envia a onResume per actualitzar la
     * temperatura mostrada cada vegada que es canvia d'activity
     */
    @Override
    protected void onResume(){
        super.onResume();
        sendCommand(STATUS);
    }

    /**
     * Resultat que es retorna despres d'editar el dispositiu. Si s'ha modificat el nom es canvia el títol
     * d'aquesta activity
     */

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent){
        super.onActivityResult(requestCode, resultCode, intent);

        if(requestCode == 2){
            device = (DeviceAC)intent.getSerializableExtra(DeviceACActivity.DEVICE);
            getSupportActionBar().setTitle(device.getName());
        }
    }

    /**
     * Crea el menu de l'activity inflant el layout menu_device
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_device, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }

        return true;
    }

    /**
     * Accions que s'executen quan es selecciona una opció del menú
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        switch (id){

            case R.id.menu_delete_device:   //Esborrar dispositiu
                Log.d("", "Dispositiu seleccionat: " + device.getName());
                deviceACViewModel.deleteDevice(device);
                finish();
                return true;

            case R.id.menu_edit_device:     //Editar dispositiu
                Log.d("", "Editar dispositiu: " + device.getName());
                editDevice();

                getSupportActionBar().setTitle(device.getName());

                return true;


            default:
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Edita el dispositiu seleccionat. Crea un nou intent amb la mateixa Activity per crear nous
     * dispositius, pero en aquest cas s'utilitza per editar el dispositiu
     */
    public void editDevice() {
        Intent intentEditDevice = new Intent(this, NewDeviceACActivity.class);

        intentEditDevice.putExtra(DeviceACActivity.DEVICE, device);
        intentEditDevice.putExtra("action", NewDeviceACActivity.EDIT_DEVICE);
        startActivityForResult(intentEditDevice, 2);

    }

    /**
     * Envia la comanda al dispositiu remot. Els paràmetres del request contenen
     * el mode de funcionament i la temperatura que es vol en les comandes per
     * arrancar l'equip i actualitzar
     * @param action
     * @return - true si s'ha pogut realitzar la connexió amb el dispositiu remot. False en cas contrari
     */
    private boolean sendCommand(int action){

        //RequestQueue requestQueue = DobileRequestQueue.getInstance(this.getApplicationContext()).getRequestQueue();

        //En cas de produir-se un error en la connexió retorna fals
        final boolean[] result = {false};

        String url = "http://" + device.getIp();
        url += createRequest(action);
        Log.d("SENDCOMMAND", "Url: " + url);

        StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                new Response.Listener<String>() {

                    @Override
                    public void onResponse(String response) {
                        //Actualitza la temperatura mostrada en el cas que sigui un request status
                        updateTemperature(response);
                        result[0] = true;
                        Log.d("SENDCOMMAND", "Resposta connexió: "+ response);
                    }
                }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Context ctx = getApplication();
                CharSequence text = "No s'ha pogut connectar amb el dispositiu remot";
                int duration = Toast.LENGTH_SHORT;
                Toast toast = Toast.makeText(ctx,text, duration);
                toast.show();;

                Log.d("SENDCOMMAND", "Error: " + error.getMessage());


            }
        });

        // Add the request to the RequestQueue.
        DobileRequestQueue.getInstance(this).addToRequestQueue(stringRequest);

        return result[0];

    }

    /**
     * Crea els paràmetres a enviar amb el request de la comanda. Cada paràmetre
     * conté l'estat dels interruptors, activat  o desactivat. Els
     *
     * Paràmetres:
     *
     *
     * @return String amb els paràmetres a enviar amb el request
     */
    private String createRequest(int action){

        String request = "";
        String mode;

        if (swMode.isChecked()){
            mode = "h";
        }else {
            mode = "c";
        }

        switch (action){
            case ON:
                request = "/c=o/t=" + npTemp.getValue() + "/m=" + mode;
                break;

            case OFF:
                request = "/c=x";
                break;

            case UPDATE:
                request = "/c=u/t=" + npTemp.getValue() + "/m=" + mode;
                break;

            case STATUS:
                request = "/status";



        }

        return request;

    }

    /**
     * Comprova que la resposta rebuda del dispositiu remot és una resposta correcta al request status
     * i en el casi que sigui aixi, actualitza la temperatura ambient mostrada
     */
    private void updateTemperature(String response){

        if ((response.length() == 9) && (response.charAt(1) == 't')){
            String temp = response.substring(3,5) + "ºC";
            textTemperature.setText(temp);
        }

    }

    /**
     * Crea un nou observer sobre el LiveData que conté el dispositiu. Quan es produeix un canvi a la base de dades
     * actualitza la vista. D'aquesta manera es manté l'estat encara que es torni enrera o es surti de l'aplicació
     */
    private void setObserver(){
        deviceACViewModel.getDevice(device.getId()).observe(this, new Observer<DeviceAC>() {
            @Override
            public void onChanged(@Nullable final DeviceAC deviceLiveData) {

                boolean on = swOn.isChecked();
                boolean mode = swMode.isChecked();
                int temperature = npTemp.getValue();

                int liveDataOn = deviceLiveData.getOn();
                int liveDataMode = deviceLiveData.getMode();
                int liveDataTemp = deviceLiveData.getTemperature();
                Log.d("DeviceACAtivity", "Observer - Darrera temperatura ajustada: " + liveDataTemp);
                Log.d("DeviceACActivity", "Observer - Darrer mode ajustat: " + liveDataMode);

                //Si s'ha modificat l'estt de l'interruptor ON/OFF es modifica la vista si és neceessari
                if(on && (liveDataOn == 0)){
                    swOn.setChecked(false);
                }else if(!on && liveDataOn == 1){
                    swOn.setChecked(true);
                }

                //Si s'ha modificat el mode de funcionament es modifica la vista si és necessari
                if (mode && (liveDataMode == 0)  ){
                    swMode.setChecked(false);
                }else if(!mode && (liveDataMode == 1)){
                    swMode.setChecked(true);
                }

                //Si s'ha mofificat la temperatura del numberpicker s'actualitza la vista si és necessari
                if (temperature != liveDataTemp){
                    npTemp.setValue(liveDataTemp);
                }
            }
        });
    }
}
