/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat que envia els comandaments dels dispositius de tipus interruptor
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;

import androidx.appcompat.view.menu.MenuBuilder;
import androidx.cardview.widget.CardView;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProviders;
import android.content.Context;
import android.content.Intent;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import androidx.appcompat.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.widget.Switch;
import android.widget.Toast;


import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;

import org.dobile.R;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;
import org.dobile.network.DobileRequestQueue;
import org.dobile.viewmodel.DeviceNameViewModel;
import org.dobile.viewmodel.DeviceSwitchViewModel;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe que envia els comandaments de un dispositiu de tipus interruptor
 */
public class DeviceSwitchActivity extends AppCompatActivity {

    private static final String LOG_TAG = DeviceSwitchActivity.class.getSimpleName();
    public static final String DEVICE = "org.dobile.ui.deviceSwitchActivity.extra.DEVICE";
    public static final String MODE = "org.dobile.ui.deviceSwitchActivity.extra.MODE";

    private final int SWITCH = 0;
    private final int STATUS = 1;

    //Entity del dispositiu i la llista d'Entitys dels noms d'interruptor
    private DeviceSwitch device;
    private List<SwitchNames> lDeviceNames = null;

    //ViewModels
    private DeviceSwitchViewModel deviceSwitchViewModel;
    private DeviceNameViewModel mDeviceNameViewModel;

    //Llistes dels CardViews i els seus interruptors i TextView amb el nom de cada interruptor
    private List<TextView> namesSwitches = new ArrayList<TextView>();
    private List<Switch> switches = new ArrayList<>();
    private List<CardView> cards = new ArrayList<>();


    /**
     * Creació de l'activity amb layout activity_device_switch i toolbar toolbarDeviceSwitch
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_device_switch);
        Toolbar toolbar = findViewById(R.id.toolbarDeviceSwitch);
        setSupportActionBar(toolbar);

        //Mostra el butó Enrera per tornar a la pantalla de l'escena
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //Listener pel butó Enrera. Finalitza l'activity
        toolbar.setNavigationOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        //Assigna els viewmodels
        deviceSwitchViewModel = ViewModelProviders.of(this).get(DeviceSwitchViewModel.class);
        mDeviceNameViewModel = ViewModelProviders.of(this).get(DeviceNameViewModel.class);

        //A través de l'intent recupera el dispositiu seleccionat i assigna el seu nom al títol
        Intent intent = getIntent();
        device = (DeviceSwitch)intent.getSerializableExtra("deviceSelected");
        Log.d(LOG_TAG, "Dispositiu seleccionat - id: " + device.getId());

        getSupportActionBar().setTitle(device.getName());

        //Crea tres llistes pels interruptors. Una llista conté els switches, una altra conté els textview amb els
        //noms dels interruptors i una tercerca conté els cardview. Cada cardview conté un switch i el seu nom
        //També assigna els liteners als switches
        setSwitches();;


         // Crea un nou Observer per la llista de noms d'interruptors
        setObserver();

    }

    /**
     * S'envia una comanda per demanar l'estat dels interruptors i actualitzar l'estat mostrat en pantalla
     */
    @Override
    protected void onResume(){
        super.onResume();
        sendCommand(STATUS);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent){
        super.onActivityResult(requestCode, resultCode, intent);

        if(requestCode == 2){
            device = (DeviceSwitch)intent.getSerializableExtra(DeviceSwitchActivity.DEVICE);
            getSupportActionBar().setTitle(device.getName());
        }
    }

    /**
     * Crea el menu de l'activity inflant el layout menu_device
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_device, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }

        return true;
    }

    /**
     * Accions que s'executen quan es selecciona una opció del menú
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        switch (id){

            case R.id.menu_delete_device:   //Esborrar dispositiu
                Log.d("", "Dispositiu seleccionat: " + device.getName());
                deviceSwitchViewModel.deleteDevice(device);
                finish();
                return true;

            case R.id.menu_edit_device:     //Editar dispositiu
                Log.d("", "Editar dispositiu: " + device.getName());
                editDevice();
                //Falta actualitzar el nom de la vista despres d'editar

                getSupportActionBar().setTitle(device.getName());

                return true;


            default:
                return super.onOptionsItemSelected(item);
        }


    }

    /**
     * Edita el dispositiu seleccionat. Crea un nou intent amb la mateixa Activity per crear nous
     * dispositius, pero en aquest cas s'utilitza per editar el dispositiu
     */
    public void editDevice() {
        Intent intentEditDevice = new Intent(this, NewDeviceSwitchActivity.class);

        if (lDeviceNames != null){
            device.setSwitchNames(lDeviceNames);
        }
        intentEditDevice.putExtra(DeviceSwitchActivity.DEVICE, device);
        intentEditDevice.putExtra("action", NewDeviceSwitchActivity.EDIT_DEVICE);
        startActivityForResult(intentEditDevice, 2);

    }

    /**
     * Crea les llistes d'interruptors, dels textview que contenen els noms de cardview que contenen un interruptor i
     * el seu nom. Assigna listeners als interruptors. Quan es modifica un interruptor s'envia automàticament la
     * comanda al dispositiu
     */
    private void setSwitches(){
        //Crea la llista de TextView que mostraran el nom dels interruptors
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch1));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch2));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch3));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch4));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch5));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch6));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch7));
        namesSwitches.add((TextView)findViewById(R.id.nameSwitch8));

        //Crea la llista de Switches per controlar la connexió dels interruptors
        switches.add((Switch)findViewById(R.id.switch1));
        switches.add((Switch)findViewById(R.id.switch2));
        switches.add((Switch)findViewById(R.id.switch3));
        switches.add((Switch)findViewById(R.id.switch4));
        switches.add((Switch)findViewById(R.id.switch5));
        switches.add((Switch)findViewById(R.id.switch6));
        switches.add((Switch)findViewById(R.id.switch7));
        switches.add((Switch)findViewById(R.id.switch8));

        //Crea la llista dels CardViews que contenen cada interruptor i els seus TextView amb el nom
        cards.add((CardView)findViewById(R.id.cardView1));
        cards.add((CardView)findViewById(R.id.cardView2));
        cards.add((CardView)findViewById(R.id.cardView3));
        cards.add((CardView)findViewById(R.id.cardView4));
        cards.add((CardView)findViewById(R.id.cardView5));
        cards.add((CardView)findViewById(R.id.cardView6));
        cards.add((CardView)findViewById(R.id.cardView7));
        cards.add((CardView)findViewById(R.id.cardView8));

        //Assigna un listener a cada switch. Cuan es modifica un s'envia la comanda per activar o
        //desactivar els interruptors
        for (Switch swt: switches){
            swt.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    sendCommand(SWITCH);
                }
            });
        }
    }

    /**
     * Crea un nou Observer per la llista de noms d'interruptors. Quan es produeix un canvi a la base de dades
     * actualitza els noms d'interruptors i fa invisibles els que no es necessiten o visibles els que s'afegeixen
     */
    private void setObserver(){

        final Observer<List<SwitchNames>> observer = new Observer<List<SwitchNames>>(){

            /**
             * onChanged s'executa quan l'Observer detecta un canvi a la llista de noms d'interruptors
             * Si el nombre de interruptors canvia, s'afegeixen nous interruptors o s'eliminen interruptors,
             * es modifica la visibilitat dels TextView i Switch segons si són necessaris o no
             *
             * @param devNames Llista dels noms d'interruptors
             */
            public void onChanged(@Nullable final List<SwitchNames> devNames){

                if (devNames != null){
                    lDeviceNames = devNames;
                    for (int i = 0; i < devNames.size(); i++){
                        TextView tview =  namesSwitches.get(i);
                        Switch sw = switches.get(i);
                        CardView card = cards.get(i);
                        tview.setText(devNames.get(i).getName());
                        Log.d(LOG_TAG, "Observer - devNames deviceId: " + devNames.get(i).getDeviceId());

                        //En el cas que s'hagin afegit nous interruptors es fan visibles els que es necessiti
                        if(tview.getVisibility() == View.INVISIBLE){
                            tview.setVisibility(View.VISIBLE);
                            sw.setVisibility(View.VISIBLE);
                            card.setVisibility(View.VISIBLE);
                        }
                    }

                    //La resta d'interruptors que no s'empren es fan invisibles
                    for (int i = devNames.size(); i < 8;i++){
                        namesSwitches.get(i).setVisibility(View.INVISIBLE);
                        switches.get(i).setVisibility(View.INVISIBLE);
                        cards.get(i).setVisibility(View.INVISIBLE);
                    }
                }else {
                    Log.d(LOG_TAG, "Observer - devNames null");
                }

            }
        };

        //Obté el LiveData amb la llista de noms d'interruptors i li assigna l'Observer
        LiveData<List<SwitchNames>> lDevNames = mDeviceNameViewModel.getSwNamesForDevice(device.getId());
        if (lDevNames == null){
            Log.d(LOG_TAG, "Llista DeviceNamess null");
        }else {
            Log.d(LOG_TAG, "Llista DeviceNames no null");
            lDevNames.observe(this, observer);
        }
    }

    /**
     * Envia la comanda al dispositiu remot. Els paràmetres del request contenen
     * l'estat dels interruptors
     */
    private void sendCommand(int action){

        RequestQueue requestQueue = DobileRequestQueue.getInstance(this.getApplicationContext()).getRequestQueue();

        String url = "http://" + device.getIp();
        url +=  createRequest(action);
        Log.d("SENDCOMMAND", "Url: " + url);

        StringRequest stringRequest = new StringRequest(Request.Method.GET, url,
                new Response.Listener<String>() {
                    @Override
                    public void onResponse(String response) {
                        updateStatus(response);
                        Log.d("SENDCOMMAND", "Resposta connexió: "+ response);
                    }
                }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Context ctx = getApplication();
                CharSequence text = "No s'ha pogut connectar amb el dispositiu remot";
                int duration = Toast.LENGTH_SHORT;
                Toast toast = Toast.makeText(ctx,text, duration);
                toast.show();;

                Log.d("SENDCOMMAND", "Error: " + error.getMessage());


            }
        });

        // Add the request to the RequestQueue.
        DobileRequestQueue.getInstance(this).addToRequestQueue(stringRequest);

    }

    /**
     * Crea els paràmetres a enviar amb el request de la comanda. Cada paràmetre
     * conté l'estat dels interruptors, activat  o desactivat. Els
     *
     * Paràmetres:
     *      s1 - interruptor 1
     *      s2 - interruptor 2
     *      s3 - interruptor 3
     *      s4 - interruptor 4
     *      s5 - interruptor 5
     *      s6 - interruptor 6
     *      s7 - interruptor 7
     *      s8 - interruptor 8
     *
     * Valors dels paràmetres:
     *      0 - desactivat (OFF)
     *      1 - activat    (ON)
     *
     * @return String amb els paràmetres a enviar amb el request
     */
    private String createRequest(int action){

        String request = "";

        switch(action){
            case  SWITCH:
                for(int i = 1; i <= device.getNumSwitches();i++){
                    Switch sw = switches.get(i-1);
                    if ((sw.isChecked())){
                        request += "/s" + i + "=1";
                    }else{
                        request += "/s" + i + "=0";
                    }

                }
                break;

            case STATUS:
                request = "/status";

        }



        return request;

    }

    /**
     * Comprova que la resposta rebuda del dispositiu remot és una resposta al request status
     * i en el casi que sigui aixi, actualitza l'estat dels interruptors mostrat en pantalla.
     * En el cas d'estar torna enrera o que l'app estigui inactiva, al tornar a primer pla no coincideix l'estat
     * mostrat dels interruptors en pantalla amb l'estat real. Per tant és necessari actualitzar
     */
    private void updateStatus(String response){

       int lenRequest = response.length();
       int numSwitches = device.getNumSwitches();

       if (response.contains("s1")){
            int indexReq = 4;    //posicio del caracter que indica l'estat en el request
            int indexSwitch = 0;      //numero d'interruptor de la llista d'interruptors

            while((indexReq < lenRequest) && (indexSwitch < numSwitches) ){

                char swState = response.charAt(indexReq);   //caracter del request per cada interruptor. 0 apagat, 1 ences

                if (swState == '0'){
                    if (switches.get(indexSwitch).isChecked()){
                        switches.get(indexSwitch).setChecked(false);
                    }
                }else{
                    if (!switches.get(indexSwitch).isChecked()){
                        switches.get(indexSwitch).setChecked(true);
                    }

                }

                indexReq  = indexReq + 4;
                indexSwitch++;

            }
        }

    }


}
