/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat principal que mostra els escenaris creats, amb les opcions per crear, esborrar i editar escenaris
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.ui;

import androidx.appcompat.view.menu.MenuBuilder;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProviders;
import android.content.DialogInterface;
import android.os.Bundle;
import androidx.annotation.Nullable;

import com.google.android.material.floatingactionbutton.ExtendedFloatingActionButton;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.appcompat.widget.Toolbar;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.view.Window;
import android.widget.EditText;
import android.widget.Toast;

import org.dobile.R;
import org.dobile.db.entity.Scene;
import org.dobile.viewmodel.SceneViewModel;

import java.lang.reflect.Method;
import java.util.List;

/**
 * Classe principal que mostra la llista d'escenes
 */
public class MainActivity extends AppCompatActivity{

    private static final String LOG_TAG = MainActivity.class.getSimpleName();

    /*ViewModel per interactuar amb la capa de la base dades*/
    private SceneViewModel mSceneViewModel;

    private EditText textDialog;
    public static final String SCENE_SELECTED = "org.dobile.ui.mainactivity.extra.MESSAGE";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);


        //Butó per crear noves escenes
        ExtendedFloatingActionButton fab = findViewById(R.id.fabAddScene);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                showNewSceneDialog();
            }
        });

        //RecyclerView que mostra la llista d'escenes
        RecyclerView recyclerView = findViewById(R.id.scenerecview);

        //Adapter del RecyclerView. Connecta les dades amb el View
        final SceneListAdapter adapter = new SceneListAdapter(this);
        recyclerView.setAdapter(adapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        mSceneViewModel = ViewModelProviders.of(this).get(SceneViewModel.class);

        //Crea un nou observer per la llista de totes les escenes
        setObserver(adapter);

        //Mostra la pantalla d'ajuda en el cas que no s'hagi creat cap escena encara
        showHelp();

    }

    /**
     * Crea el menu de la vista principal
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }

        return true;
    }

    /**
     * Accions que duen a terme quan s'eligeix una opció del menú
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        switch (id){
            case R.id.action_add_scene:     //Mostra el diàleg per crear una nova escena
                showNewSceneDialog();
                return true;

            case R.id.menu_main_help:       //Mostra l'ajuda
                DialogFragment helpFragment = new HelpFragment();
                helpFragment.show(getSupportFragmentManager(), "help");
                return true;


            default:
                return super.onOptionsItemSelected(item);
        }

    }




    /**
     * Mostra el diàleg per crear una nova escena
     */
    public void showNewSceneDialog() {
        // get prompts.xml view
        LayoutInflater layoutInflater = LayoutInflater.from(MainActivity.this);
        View viewDialog = layoutInflater.inflate(R.layout.dialog_new_scene, null);
        AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(MainActivity.this);
        alertDialogBuilder.setView(viewDialog);

        final EditText editText = (EditText) viewDialog.findViewById(R.id.dialogEdit);
        // setup a dialog window
        alertDialogBuilder.setCancelable(false)
                .setPositiveButton("Guardar", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        String sceneName = editText.getText().toString();
                        Log.d(LOG_TAG, sceneName);

                        if (sceneName.matches("")){
                            Toast.makeText(MainActivity.this, "No s'ha pogut crear l'escena. El nom és obligatori", Toast.LENGTH_LONG).show();

                        }else{
                            //Inserta la nova escena a la base de dades mitjançant el ViewModel
                            mSceneViewModel.insertScene(sceneName);
                        }


                    }
                })
                .setNegativeButton("Cancel·lar", //butó cancelar
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                dialog.cancel();
                            }
                        });

        // create an alert dialog
        AlertDialog alert = alertDialogBuilder.create();
        alert.setTitle("Nova escena");
        alert.show();

    }

    /**
     * Mostra una pantalla d'ajuda la primera vegada que s'inicia l'app
     */
    private void showHelp(){

        Boolean isFirstRun = getSharedPreferences("PREFERENCE", MODE_PRIVATE)
                .getBoolean("isFirstRun", true);

        if (isFirstRun) {
            DialogFragment helpFragment = new HelpFragment();
            helpFragment.show(getSupportFragmentManager(), "help");
        }

        getSharedPreferences("PREFERENCE", MODE_PRIVATE).edit()
                .putBoolean("isFirstRun", false).commit();
    }

    /**
     * Crea un nou observer per la llista de totes les escenes. Qualssevol canvi a la base
     * de dades actualitza automàticament el View
     */
    private void setObserver(final SceneListAdapter adapter){

        mSceneViewModel.getAllScenes().observe(this, new Observer<List<Scene>>() {
            @Override
            public void onChanged(@Nullable final List<Scene> scenes) {
                // Actualitza la llista d'escenes
                adapter.setScenes(scenes);
            }
        });

        Log.d(LOG_TAG, "Oncreate nombre items: " + adapter.getItemCount());
    }



}
