

/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat per crear o editar dispositius de tipus aire acondicionat
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;

import androidx.appcompat.view.menu.MenuBuilder;
import androidx.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.Toast;

import org.dobile.R;
import org.dobile.db.entity.DeviceAC;
import org.dobile.viewmodel.DeviceSwitchViewModel;
import org.dobile.viewmodel.DeviceACViewModel;

/**
 * Classe Activity per crear o editar dispositius
 */
public class NewDeviceACActivity extends AppCompatActivity {

    /*ViewModels dels dispositius i noms d'interruptors*/
    private DeviceSwitchViewModel mDeviceSwitchViewModel;
    private DeviceACViewModel mDeviceACViewModel;

    /*Entitys*/
    private DeviceAC deviceAC;
    private static final String LOG_TAG = NewDeviceACActivity.class.getSimpleName();

    private static final int EDIT = 0;
    private static final int CREATE = 1;

    private int action = 0;

    public static final int NEW_DEVICE = 0;
    public static final int EDIT_DEVICE = 1;


    private String name;
    private String ip;
    private int sceneId;

    /*EditText pel nom*/
    private EditText edName;

    /*EditText per la IP*/
    private EditText edIP;

    /**
     * Crea l'activity amb el layout activity_new_ac_device i el toolbar toolbar
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_ac_device);
        Toolbar toolbar = findViewById(R.id.toolbar);

        edName =  findViewById(R.id.editNameACDevice);
        edIP = findViewById(R.id.editACIP);


        //Listener assignats quan es canvia el focus als EditText del nom i la IP
        //Assigna validadors per comprovar que l'entrada de text no té caràcters no
        //permesos en el cas del nom, i té el format correcte per la IP
        edName.setOnFocusChangeListener(new Validator());
        edIP.setOnFocusChangeListener(new Validator());



        mDeviceACViewModel = ViewModelProviders.of(this).get(DeviceACViewModel.class);

        setSupportActionBar(toolbar);

        //Recupera l'intent que ha arrancat l'Activity i realitza les accions corresponents segons es tracti de
        //crear un nou dispositiu o editar un dispositiu existent
        getIntentInfo();


    }




    /**
     * Crea un nou dispositiu de tipus aire acondicionat
     *
     * @return true si s'ha creat el dispositiu amb èxit, fals en cas d'errors
     */
    private boolean createDevice(){
        //Comprova si existeixen errors. Si hi ha errors avisa i no continua
        if (!notNullFields()) {
            Toast.makeText(this, "Existeixen errors", Toast.LENGTH_LONG).show();
            return false;
        }

        name = this.edName.getText().toString();
        ip = this.edIP.getText().toString();

        DeviceAC device = new DeviceAC(name, ip, 0, 22, sceneId);
        mDeviceACViewModel.insertDevice(device);

        return true;

    }

    /**
     * Edita un dispositiu de tipus aire acondicionat
     *
     * @param device Dispositiu seleccionat
     * @return true si s'ha editat amb èxit, fals en cas d'errors
     */
    private boolean editACDevice(DeviceAC device){
        name = this.edName.getText().toString();
        ip = this.edIP.getText().toString();


        //Comprova si existeixen errors. Si hi ha errors avisa i no continua
        if (!notNullFields()) {
            Toast.makeText(this, "Existeixen errors", Toast.LENGTH_LONG).show();
            return false;
        }

        device.setName(name);
        device.setIp(ip);

        mDeviceACViewModel.update(device);
        return true;
    }



    /**
     * Crea el menú amb el layout menu_new_activity
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_new_activity, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }

        return true;
    }

    /**
     * Accions a executar quan es selecciona una opció del menu
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();
        Intent intent;

        switch (id){
            case NewDeviceACActivity.EDIT_DEVICE:
                if (editACDevice(deviceAC)) finish();
                return true;

            case R.id.action_device_cancel: //Cancellar
                intent = new Intent();
                intent.putExtra(DeviceACActivity.DEVICE, deviceAC);
                setResult(0, intent);
                finish();
                return true;

            case R.id.action_device_done:   //Butó fet. Es crea o edita un dispositiu de tipus aire acondicionat
                switch(action){

                    case NewDeviceACActivity.EDIT_DEVICE:

                        if (editACDevice(deviceAC)){
                            intent = new Intent();

                            intent.putExtra(DeviceACActivity.DEVICE, deviceAC);
                            setResult(2, intent);
                            finish();
                        }
                        return true;

                    case NewDeviceACActivity.NEW_DEVICE:

                        if (createDevice()) finish();
                        return true;
                }

            default:
                return super.onOptionsItemSelected(item);
        }


    }

    /**
     * Recupera la informació continguda en l'intent que ha arrancat l'activity. La mateixa activity serveix
     * per crear nous dispositius i per editar dispositius existents.
     * Si es tracta de crear un nou dispositiu es recupera la id de l'escena al que pertany.
     * En el cas que es tracti d'editar un dispositiu existent es recupera el dispositiu i s'assignen
     * les seves dades als camps del formulari corresponents
     */
    private void getIntentInfo(){
        Intent intent = getIntent();

        action = intent.getIntExtra("action",0);

        switch (action){

            case NewDeviceACActivity.EDIT_DEVICE:

                Log.d("NewDeviceActivity", "Editar dispositiu");
                this.deviceAC = (DeviceAC)intent.getSerializableExtra(DeviceACActivity.DEVICE);

                getSupportActionBar().setTitle(deviceAC.getName());

                edName.setText(deviceAC.getName());
                edIP.setText(deviceAC.getIp());

                break;

            case NewDeviceACActivity.NEW_DEVICE:

                sceneId = intent.getIntExtra(MainActivity.SCENE_SELECTED, 0);
                break;
        }
    }

    /**
     * Comprova si els camps dels formularis tener error o no
     *
     * @return true - si no hi ha errors
     *         false - si al menys un dels camps té error
     */
    private boolean notNullFields(){
        String textName = edName.getText().toString();
        String textIP = edIP.getText().toString();
        CharSequence strError = edName.getError();

        if ((strError != null) || (textName.matches("")) ) {
            return false;
        }

        strError = edIP.getError();
        if ((strError != null) || (textIP.matches("")) ) {
            return false;
        }


        //No hi ha cap error. Tots els camps del formulari són correctes
        return true;
    }


}
