/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Activitat per crear o editar dispositius
 *
 * @author Macià Sintes Andreu
 **/


package org.dobile.ui;

import androidx.appcompat.view.menu.MenuBuilder;
import androidx.lifecycle.ViewModelProviders;
import android.content.Intent;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import org.dobile.R;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;
import org.dobile.viewmodel.DeviceNameViewModel;
import org.dobile.viewmodel.DeviceSwitchViewModel;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe Activity per crear o editar dispositius
 */
public class NewDeviceSwitchActivity extends AppCompatActivity {

    public static final int NEW_DEVICE = 0 ;
    public static final int EDIT_DEVICE =  1;

    /*ViewModels dels dispositius i noms d'interruptors*/
    private DeviceSwitchViewModel mDeviceSwitchViewModel;
    private DeviceNameViewModel mDeviceNameViewModel;

    /*Entitys*/
    private DeviceSwitch deviceSwitch;
    private List<SwitchNames> devNames;
    private static final String LOG_TAG = NewDeviceSwitchActivity.class.getSimpleName();


    private String name;
    private String ip;
    private int sceneId;
    private int numSwitches;
    private List<String> namesSwitches;

    private int action = 0;

    /*EditText pel nom*/
    private EditText edName;

    /*EditText per la IP*/
    private EditText edIP;

    /*Spinner per seleccionar el número de interruptors*/
    private Spinner spNumSwitches;

    /*Llista amb els EditText dels noms d'interruptors*/
    private List<EditText> edNamesSwitches = new ArrayList<>();


    /**
     * Crea l'activity amb el layout activity_new_switch_device i el toolbar toolbar
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_new_switch_device);
        Toolbar toolbar = findViewById(R.id.toolbar);

        edName =  findViewById(R.id.editNameSwitchDevice);
        edIP = findViewById(R.id.editSwitchIP);
        spNumSwitches = findViewById(R.id.spinner_num_devices);

        //Listener del spinner amb el número de switches. Agafa el número seleccionat
        spNumSwitches.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                int num = new Integer(spNumSwitches.getSelectedItem().toString());
                setEnabledNames(num);
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }


        });

        //Listener assignats quan es canvia el focus als EditText del nom i la IP
        //Assigna validadors per comprovar que l'entrada de text no té caràcters no
        //permesos en el cas del nom, i té el format correcte per la IP
        edName.setOnFocusChangeListener(new Validator());
        edIP.setOnFocusChangeListener(new Validator());


        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch1));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch2));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch3));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch4));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch5));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch6));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch7));
        edNamesSwitches.add((EditText)findViewById(R.id.editSwitch8));


        devNames = new ArrayList<>(8);
        setEnabledNames(1);

        mDeviceSwitchViewModel = ViewModelProviders.of(this).get(DeviceSwitchViewModel.class);

        setSupportActionBar(toolbar);

        //Recupera l'intent que ha arrancat l'Activity i realitza les accions corresponents segons si es tracta
        //de crear un nou dispositiu o d'editar un dispositiu existent
        getIntentInfo();

    }



    /**
     * Crea un nou dispositiu
     *
     * @return true si s'ha creat el dispositiu amb èxit, fals en cas d'errors
     */
    private boolean createDevice(){
        //Comprova si existeixen errors. Si hi ha errors avisa i no continua
        if (!notNullFields()) {
            Toast.makeText(this, "Existeixen errors", Toast.LENGTH_LONG).show();
            return false;
        }

        name = this.edName.getText().toString();
        ip = this.edIP.getText().toString();

        numSwitches = new Integer(this.spNumSwitches.getSelectedItem().toString());
        namesSwitches = new ArrayList<>(numSwitches);
        for (int i = 0; i < numSwitches; i++){
            String name = edNamesSwitches.get(i).getText().toString();
            if (TextUtils.isEmpty(name)){
                namesSwitches.add(edNamesSwitches.get(i).getHint().toString());
            }else {
                namesSwitches.add(name);
            }
        }
        DeviceSwitch device = new DeviceSwitch(name, ip, numSwitches, sceneId);
        List<SwitchNames> switchNames = new ArrayList<>();

        for (int i = 0; i < namesSwitches.size(); i++){
            SwitchNames devName = new SwitchNames(namesSwitches.get(i));
            switchNames.add(devName);
        }

        device.setSwitchNames(switchNames);
        mDeviceSwitchViewModel.insertDevice(device);
        return true;


    }

    /**
     * Edita un dispositiu de tipus interruptor
     *
     * @param dev Dispositiu seleccionat
     * @return true si s'ha editat amb èxit, fals en cas d'errors
     */
    private boolean editSwitchDevice(DeviceSwitch dev){
        name = this.edName.getText().toString();
        ip = this.edIP.getText().toString();


        //Comprova si existeixen errors. Si hi ha errors avisa i no continua
        if (!notNullFields()) {
            Toast.makeText(this, "Existeixen errors", Toast.LENGTH_LONG).show();
            return false;
        }

        dev.setName(name);
        dev.setIp(ip);


        numSwitches = new Integer(this.spNumSwitches.getSelectedItem().toString());
        namesSwitches = new ArrayList<>(numSwitches);
        for (int i = 0; i < numSwitches; i++){
            String name = edNamesSwitches.get(i).getText().toString();
            if (TextUtils.isEmpty(name)){
                namesSwitches.add(edNamesSwitches.get(i).getHint().toString());
            }else {
                namesSwitches.add(name);
            }
        }


        dev.setNumSwitches(numSwitches);
        this.deviceSwitch = dev;

        mDeviceSwitchViewModel.update(dev, namesSwitches);


        return true;

    }



    /**
     * Crea el menú amb el layout menu_new_activity
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_new_activity, menu);

        // To display icon on overflow menu
        if(menu instanceof MenuBuilder){
            MenuBuilder m = (MenuBuilder) menu;
            m.setOptionalIconsVisible(true);
        }

        return true;
    }

    /**
     * Accions a executar quan es selecciona una opció del menu
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();
        Intent intent;

        switch (id){
            case R.id.action_device_cancel: //Cancellar
                intent = new Intent();
                intent.putExtra(DeviceSwitchActivity.DEVICE, deviceSwitch);
                setResult(0, intent);
                finish();
                return true;

            case R.id.action_device_done:   //Butó fet. Es crea un dispositiu
                switch(action){

                    case NewDeviceSwitchActivity.EDIT_DEVICE:

                        if (editSwitchDevice(deviceSwitch)) {
                            intent = new Intent();

                            intent.putExtra(DeviceSwitchActivity.DEVICE, deviceSwitch);
                            setResult(2, intent);
                            finish();
                        }
                        return true;

                    case NewDeviceSwitchActivity.NEW_DEVICE:

                        if (createDevice()) finish();
                        return true;
                }


            default:
                return super.onOptionsItemSelected(item);
        }


    }

    /**
     * Recupera la informació continguda en l'intent que ha arrancat l'activity. La mateixa activity serveix
     * per crear nous dispositius i per editar dispositius existents.
     * Si es tracta de crear un nou dispositiu es recupera la id de l'escena al que pertany.
     * En el cas que es tracti d'editar un dispositiu existent es recupera el dispositiu i s'assignen
     * les seves dades als camps del formulari corresponents
     */
    private void getIntentInfo(){

        Intent intent = getIntent();

        action = intent.getIntExtra("action",0);

        switch (action){

            case NewDeviceSwitchActivity.EDIT_DEVICE:

                Log.d("NewDeviceActivity", "Editar dispositiu");
                this.deviceSwitch = (DeviceSwitch)intent.getSerializableExtra(DeviceSwitchActivity.DEVICE);

                getSupportActionBar().setTitle(deviceSwitch.getName());

                edName.setText(deviceSwitch.getName());
                edIP.setText(deviceSwitch.getIp());


                spNumSwitches.setSelection(deviceSwitch.getNumSwitches()- 1);

                for (int i = 0; i < deviceSwitch.getNumSwitches(); i++){
                    edNamesSwitches.get(i).setText(deviceSwitch.getSwitchNames().get(i).getName());

                }

                break;

            case NewDeviceSwitchActivity.NEW_DEVICE:

                sceneId = intent.getIntExtra(MainActivity.SCENE_SELECTED, 0);
                break;
        }

    }

    /**
     * Comprova si els camps dels formularis tener error o no
     *
     * @return true - si no hi ha errors
     *         false - si al menys un dels camps té error
     */
    private boolean notNullFields(){

        String textName = edName.getText().toString();
        String textIP = edIP.getText().toString();
        CharSequence strError = edName.getError();

        if ((strError != null) || (textName.matches("")) ) {
            return false;
        }

        strError = edIP.getError();
        if ((strError != null) || (textIP.matches("")) ) {
            return false;
        }

        for (int i = 0; i < numSwitches; i++){
            strError = edNamesSwitches.get(i).getError();
            if (strError != null ) {
                return false;
            }
        }

        //No hi ha cap error. Tots els camps del formulari són correctes
        return true;
    }

    /**
     * Activa o desactiva els editText dels noms de dispositius segons la quantitat de dispositius
     * seleccionats
     * @param position - posició de la llista de EditText a partir de la qual es desactiven
     */
    private void setEnabledNames(int position){

        for (int i = 0; i < 8; i++){
            //for (EditText editText: edNamesSwitches){
            EditText editText = edNamesSwitches.get(i);
            Log.d("NewDeviceActivity", "setEnabledNames: i = " + i);
            Log.d("NewDeviceActivity", "setEnabledNames: position = " + position);
            if (i < position){
                Log.d("NewDeviceActivity", "i < position - Enabled");
                editText.setEnabled(true);
                if (!editText.isEnabled()){
                    editText.setEnabled(true);
                }
            }else{
                Log.d("NewDeviceActivity", "i >= position - Disabled");
                if (editText.isEnabled()) {
                    editText.setEnabled(false);
                }
            }

        }
    }
}
