/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * Validador pels camps dels formularis
 *
 * @author Macià Sintes Andreu
 */
package org.dobile.ui;


import android.text.TextUtils;
import android.util.Patterns;
import android.view.View;
import android.widget.EditText;

import org.dobile.R;

/**
 * Validador pels camps dels formularis. S'activa quan canvia el focus de l'EditText
 */
public class Validator implements View.OnFocusChangeListener {


    /**
     * Comprova un EditText i avisa si hi ha errors assignant un missatge d'avis
     * @param editText
     * @return
     */
    public boolean validate(EditText editText){

        String text = editText.getText().toString();

        //En el camp per entrar IPs que no estigui buit i que tingui el format de IP correcte
        if(editText.getId() == R.id.editSwitchIP){

            if(TextUtils.isEmpty(text)){
                editText.setError(editText.getHint() + " és necessari");
                return false;
            }else if ( !Patterns.IP_ADDRESS.matcher(text).matches()){
                editText.setError(editText.getHint() + " no té el format correcte. Pe exemple 192.168.1.256");
                return  false;
            }else{
                return true;
            }
        }

        //En el camp per entrar IPs que no estigui buit i que tingui el format de IP correcte
        if(editText.getId() == R.id.editACIP){

            if(TextUtils.isEmpty(text)){
                editText.setError(editText.getHint() + " és necessari");
                return false;
            }else if ( !Patterns.IP_ADDRESS.matcher(text).matches()){
                editText.setError(editText.getHint() + " no té el format correcte. Pe exemple 192.168.1.256");
                return  false;
            }else{
                return true;
            }
        }

        //Per la resta de camps es comprova que els EditText no estiguin buits i avisa en cas contrari
       if(TextUtils.isEmpty(text)){
            editText.setError(editText.getHint() + " és necessari");
            return false;
        }else {
            editText.setError(null);
            return true;
        }


    }

    /**
     * S'activa la validació quan canvia el focus de l'EditText
     *
     * @param view
     * @param hasFocus
     */
    @Override
    public void onFocusChange(View view, boolean hasFocus) {
        if (!hasFocus){
            validate((EditText) view);
        }

    }
}
