/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * L'app utilitza el patrò Model-View-ViewModel recomanat per Android. Aquest patrò està compost de:
 *
 *  - View: La interfície gràfica que interactua amb l'usuari
 *  - ViewModel: Connecta les dades amb la interfície gràfica
 *  - DataModel: La capa de dades, en aquest cas la base de dades
 *
 *  ViewModel pels dispositius de tipus aire acondicionat
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.viewmodel;

import android.app.Application;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;

import org.dobile.db.DobileRepository;
import org.dobile.db.entity.DeviceAC;

import java.util.List;

/**
 * Classe que implementa la capa ViewModel pels dispositius de tipus aire acondicionat
 */
public class DeviceACViewModel extends AndroidViewModel {

    private static final String LOG_TAG = DeviceACViewModel.class.getSimpleName();

    /*Repositori de la base de dades*/
    private DobileRepository mRepository;

    /*Llista de tots els dispositius de tipus aire acondicionat*/
    private LiveData<List<DeviceAC>> mAllACDevices;

    /**
     * Constructor. Crea el repositori i obté la llista de tots els dispositius de tipus aire acondicionat
     *
     * @param application
     */
    public DeviceACViewModel (Application application) {
        super(application);
        mRepository = new DobileRepository(application);

        mAllACDevices = mRepository.getAllACDevices();

    }


    /**
     * Retorna la llista de tots els dispositius de tipus aire acondicionat
     *
     * @return LiveData amb tots els dispositius de tipus aire acondicionat
     */
    public LiveData<List<DeviceAC>> getAllDevices(){
        return mAllACDevices;
    }

    /**
     * Inserta un nou dispositiu de tipus aire acondicionat
     *
     * @param device
     */
    public void insertDevice(DeviceAC device) {


        mRepository.insertDeviceAC(device);

    }


    /**
     * Esborra un dispositiu de tipus aire acondicionat
     *
     * @param device
     */
    public void deleteDevice(DeviceAC device) {
        mRepository.deleteDeviceAC(device);
    }

    /**
     * Actualitza un dispositiu de tipus aire acondicioant
     * @param device
     */
    public void update(DeviceAC device){
        mRepository.updateDeviceAC(device);
    }

    /**
     * Retorna els dispositius de tipus aire acondicionat d'una escena
     *
     * @param sceneId
     * @return LiveData amb la llista de dispositius de una escena
     */
    public LiveData<List<DeviceAC>> getDevicesForScene(int sceneId){
        LiveData<List<DeviceAC>> lDevices;

        lDevices = mRepository.getDevicesACForScene(sceneId);

        return  lDevices;
    }

    /**
     * Retorna un dispositiu encapsulat en un LiveData
     * @param idDevice - Id del dispositiu
     * @return - Dispositiu aire acondicionat encapsulat en un LiveData
     */
    public LiveData<DeviceAC> getDevice(int idDevice) {
        LiveData<DeviceAC> device = mRepository.getDeviceAC(idDevice);
        return  device;
    }
}
