/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * L'app utilitza el patrò Model-View-ViewModel recomanat per Android. Aquest patrò està compost de:
 *
 *  - View: La interfície gràfica que interactua amb l'usuari
 *  - ViewModel: Connecta les dades amb la interfície gràfica
 *  - DataModel: La capa de dades, en aquest cas la base de dades
 *
 *  ViewModel pels noms de dispositius
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.viewmodel;

import android.app.Application;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;

import org.dobile.db.DobileRepository;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;

import java.util.List;

/**
 * Classe que implementa la capa ViewModel pels noms d'interruptors
 */
public class DeviceNameViewModel extends AndroidViewModel {

    /*Repositori de la base de dades*/
    private DobileRepository mRepository;

    /*LiveData que conté la llista dels noms d'interruptor*/
    private LiveData<List<SwitchNames>> lDeviceNames;

    /**
     * Constructor. Crea el repositori
     *
     * @param application
     */
    public DeviceNameViewModel (Application application) {
        super(application);
        mRepository = new DobileRepository(application);
    }

    /**
     * Retorna la llista de tots els noms d'interruptors
     *
     * @return LiveData amb la llista de tots els noms d'interruptors
     */
    public LiveData<List<SwitchNames>> getAllDevNames() {
        LiveData<List<SwitchNames>> lDevNames;
        lDevNames = mRepository.getmAllSwitchNames();

        return lDevNames;
    }

    /**
     * Retorna els noms dels interruptors d'un dispositiu
     *
     * @param deviceId
     * @return llista dels noms d'interruptors
     */
    public LiveData<List<SwitchNames>> getSwNamesForDevice(int deviceId){
        LiveData<List<SwitchNames>> lNames;

        lNames = mRepository.getSwNamesForDevice(deviceId);

        return  lNames;
    }

    /*public void insertSwitch(String name, int deviceId) {

        SwitchNames devName = new SwitchNames(name, deviceId);
        mRepository.insertDevName(devName);
    }
*/


}