/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * L'app utilitza el patrò Model-View-ViewModel recomanat per Android. Aquest patrò està compost de:
 *
 *  - View: La interfície gràfica que interactua amb l'usuari
 *  - ViewModel: Connecta les dades amb la interfície gràfica
 *  - DataModel: La capa de dades, en aquest cas la base de dades
 *
 *  ViewModel pels dispositius de tipus interruptor
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.viewmodel;

import android.app.Application;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;

import org.dobile.db.DobileRepository;
import org.dobile.db.entity.DeviceSwitch;
import org.dobile.db.entity.SwitchNames;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe que implementa la capa ViewModel pels dispositius de tipus interruptor
 */
public class DeviceSwitchViewModel extends AndroidViewModel {

    private static final String LOG_TAG = DeviceSwitchViewModel.class.getSimpleName();

    /*Repositori de la base de dades*/
    private DobileRepository mRepository;

    /*Llista de tots els dispositius de tipus interruptor*/
    private LiveData<List<DeviceSwitch>> mAllSwitchDevices;

    /**
     * Constructor. Crea el repositori i obté la llista de tots els dispositius de tipus interruptor
     *
     * @param application
     */
    public DeviceSwitchViewModel (Application application) {
        super(application);
        mRepository = new DobileRepository(application);

        mAllSwitchDevices = mRepository.getAllSwitchDevices();

    }


    /**
     * Retorna la llista de tots els dispositius de tipus interruptor
     *
     * @return LiveData amb tots els dispositius de tipus interruptor
     */
    public LiveData<List<DeviceSwitch>> getAllDevices(){
        return mAllSwitchDevices;
    }

    /**
     * Inserta un nou dispositiu de tipus interruptor
     *
     * @param device
     *
     */
    public void insertDevice(DeviceSwitch device) {

        mRepository.insertDeviceSwitch(device);

    }


    /**
     * Esborra un dispositiu de tipus interruptor
     *
     * @param device
     */
    public void deleteDevice(DeviceSwitch device) {
        mRepository.deleteDeviceSwitch(device);
    }

    /**
     * Actualitza un dispositiu de tipus interruptor
     * @param device
     * @param namesSwitches
     */
    public void update(DeviceSwitch device, List<String> namesSwitches){
        mRepository.updateDeviceSwitch(device, namesSwitches);
    }

    /**
     * Retorna els dispositius de tipus interruptor d'una escena
     *
     * @param sceneId
     * @return LiveData amb la llista de dispositius de tipus interruptor de una escena
     */
    public LiveData<List<DeviceSwitch>> getDevicesForScene(int sceneId){
        LiveData<List<DeviceSwitch>> lDevices;

        lDevices = mRepository.getDevicesSwitchForScene(sceneId);

        return  lDevices;
    }

}
