/**
 * UOC
 * Treball de Final de Grau de Informàtica - Dobile
 *
 * Es tracta d'una app que serveix per controlar aparells, com poden ser llums, persianes, motors, aire acondicionat, etc.
 * Els controla mitjançant dos tipus de dispositius construits amb Arduino. Un de tipus interruptor i un emissor de comandes
 * infrarrojes.
 *
 * El dispositiu de tipus interruptor pot connectar i desconnectar qualssevol aparell, com per exemple llums i motors. Un
 * mateix dispositiu pot controlar al mateix temps entre 1 i 8 aparells. L'usuari eligeix quants d'interruptors vol.
 *
 * Un altre tipus de dispositiu és un emissor de comandes infrarrojes. Mitjançant un receptor de comandes
 * construit amb Arduino es pot replicar qualssevol comandament a distància. En principi s'utilitzarà per
 * controlar un equip d'aire acondicionat. Encara que es podria utilitzar per controlar qualssevol equip que funcioni
 * amb comandament a distància per infrarrojos
 *
 * L'usuari crea els escenaris que vol, com poden ser Habitació principal, Cuina, Salon, etc A cada escenari pot crear els
 * dispositius que vulgui. En els dispositius de tipus interruptor pot elegir entre 1 i 8 interruptors i posar-lis un nom
 * a cada un.
 *
 * L'app utilitza el patrò Model-View-ViewModel recomanat per Android. Aquest patrò està compost de:
 *
 *  - View: La interfície gràfica que interactua amb l'usuari
 *  - ViewModel: Connecta les dades amb la interfície gràfica
 *  - DataModel: La capa de dades, en aquest cas la base de dades
 *
 *  ViewModel per les escenes
 *
 * @author Macià Sintes Andreu
 **/

package org.dobile.viewmodel;

import android.app.Application;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;

import org.dobile.db.DobileRepository;
import org.dobile.db.entity.Scene;

import java.util.List;

/**
 * Classe que implementa la capa ViewModel per les escenes
 */
public class SceneViewModel extends AndroidViewModel {

    /*Repositori de la base de dades*/
    private DobileRepository mRepository;

    /*Llista de totes les escenes*/
    private LiveData<List<Scene>> mAllScenes;

    /**
     * Constructor. Crea el repositori i obté la llista de totes les escenes
     *
     * @param application
     */
    public SceneViewModel (Application application) {
        super(application);
        mRepository = new DobileRepository(application);
        mAllScenes = mRepository.getAllScenes();

    }

    /**
     * Retorna totes les escenes
     *
     * @return LiveData amb la llista de totes les escenes
     */
    public LiveData<List<Scene>> getAllScenes() { return mAllScenes; }

    /**
     * Inserta una escena
     *
     * @param sceneName
     */
    public void insertScene(String sceneName) {
        Scene scene = new Scene(sceneName);
        mRepository.insertScene(scene);
    }

    /**
     * Esborra una escena
     *
     * @param scene
     */
    public void deleteScene(Scene scene) {
        mRepository.deleteScene(scene);
    }

    /**
     * Actualitza una escena
     *
     * @param scene
     */
    public void updateScene(Scene scene){
        mRepository.updateScene(scene);
    }

    /*public Scene getScene( int id){

        mRepository.getScene(id);
        List<Scene> scList = mRepository.getSearchScene();


        if (scList == null) {
            return null;
        }else if (scList.size() > 0) {
            return scList.get(0);
        } else {
            return null;
        }
    }*/



}
